package com.biganto.visual.roompark.presentation.screen.albums

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.AlbumsInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class AlbumsScreenPresenter @Inject constructor(
    private val interactor: AlbumsInteractor
    ,@Named(SELECTED_ALBUM_INDEX_KEY) private var selectedIndex:Int
)
    : BigantoBasePresenter<AlbumsScreen, AlbumsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> AlbumsScreenViewState.SomeError(e) }

    override fun bindIntents() {

        val fetchParents = interactor.fetchHeaderAlbums()
            .map { AlbumsScreenViewState.AlbumsListLoaded(it,selectedIndex) }

        val fetchSelected = interactor.fetchAlbumPhotos(selectedIndex)
            .map { AlbumsScreenViewState.AlbumsSelected(it)}


        val state = restoreStateObservable
            .mergeWith(fetchParents)
            .mergeWith(fetchSelected)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(AlbumsScreenViewState::class.java), AlbumsScreen::render)
    }

}